#!/usr/bin/env python

"A job candidate editing application."

import WebStack.Generic
from WebStack.Repositories.Directory import DirectoryRepository
from XSLForms.Resources.WebResources import \
    XSLFormsResource, input, output, resources, prepare_resources as xslforms_prepare_resources
import os
import libxml2dom

# Site map imports.

from WebStack.Resources.ResourceMap import MapResource
from WebStack.Resources.Selectors import EncodingSelector
from WebStack.Resources.Static import DirectoryResource

# Configuration setting.

encoding = "utf-8"

# Resource classes.

class AdminResource(XSLFormsResource):

    "A resource providing administration facilities for job candidate profiles."

    resource_dir = resources(__file__)
    template_resources = {
        "admin" : output("admin_template.xhtml")
        }
    init_resources = {
        "admin" : input("admin_template.xhtml")
        }

    def __init__(self, repository):
        self.repository = repository

    def select_activity(self, trans, form):
        form.set_activity("admin")

    def create_document(self, trans, form):
        is_new = XSLFormsResource.create_document(self, trans, form)
        trans.get_attributes()["form_is_new"] = is_new
        return is_new

    def respond_to_input(self, trans, form):

        # Get the "show" and "edit" resource paths.
        # NOTE: These should be obtained from the site map.

        vpath = trans.get_processed_virtual_path_info()
        show_path = trans.get_path_without_info() + trans.update_path(vpath, "show")
        edit_path = trans.get_path_without_info() + trans.update_path(vpath, "edit")

        # Redirect if one of the CVs is to be shown or edited.

        selectors = form.get_selectors()
        if selectors.has_key("show"):
            name = selectors["show"][0].getAttribute("name")
            trans.redirect(trans.encode_path(show_path) +
                "?name=%s" % trans.encode_path(name))
        elif selectors.has_key("edit"):
            name = selectors["edit"][0].getAttribute("name")
            trans.redirect(trans.encode_path(edit_path) +
                "?name=%s" % trans.encode_path(name))

        # Add and remove elements according to the selectors found.

        self.remove_elements(selectors.get("remove"))
        self.add_elements(selectors.get("new"), "cv", "cvs")

    def respond_to_document(self, trans, form):

        admin = form.get_document()

        # Synchronise the repository with the CVs found.

        cvs = admin.xpath("/admin/cvs")[0]
        for key in self.repository.keys():
            if key.startswith("candidate-"):
                name = key[len("candidate-"):]
                # NOTE: Apostrophes not quoted.
                if not cvs.xpath("cv[@name = '%s']" % name):
                    if trans.get_attributes()["form_is_new"]:
                        cv = admin.createElement("cv")
                        cv.setAttribute("name", name)
                        cvs.appendChild(cv)
                    else:
                        del self.repository[key]

class CandidateUtils:

    "Methods used by candidate-related resources."

    def __init__(self, repository):
        self.repository = repository

    def select_activity(self, trans, form):
        form.set_activity("candidate")

    def create_document(self, trans, form):
        documents = form.get_documents()
        fields = trans.get_fields_from_path()
        name = fields.get("name", [u"None"])[0]

        # Ensure the presence of a document.

        if documents.has_key("candidate"):
            form.set_document(documents["candidate"])
        else:
            if self.repository is None or not self.repository.has_key("candidate-%s" % name):
                form.set_document(form.new_instance("candidate"))
            else:
                form.set_document(libxml2dom.parseString(self.repository["candidate-%s" % name]))

    def init_document(self, trans, form):
        status_xml = self.prepare_document("status")
        XSLFormsResource.init_document(self, trans, form, references={"status" : status_xml})

class DisplayResource(CandidateUtils, XSLFormsResource):

    "A resource providing editing facilities for a job candidate profile."

    resource_dir = resources(__file__)
    template_resources = {
        "candidate" : output("candidate_display_template.xhtml")
        }
    init_resources = {
        "candidate" : input("candidate_template.xhtml")
        }
    document_resources = {
        "status" : "candidate_status.xml"
        }

class CandidateResource(CandidateUtils, XSLFormsResource):

    "A resource providing editing facilities for a job candidate profile."

    resource_dir = resources(__file__)
    template_resources = {
        "candidate" : output("candidate_template.xhtml")
        }
    init_resources = {
        "candidate" : input("candidate_template.xhtml")
        }
    document_resources = {
        "status" : "candidate_status.xml"
        }

    def respond_to_input(self, trans, form):

        # Add and remove elements according to the selectors found.

        selectors = form.get_selectors()
        self.remove_elements(selectors.get("remove"))
        self.add_elements(selectors.get("add-skill"), "skill", "skills")
        self.add_elements(selectors.get("add-qualification"), "qualification", "qualifications")
        self.add_elements(selectors.get("add-employment"), "employment", "experience")

    def respond_to_document(self, trans, form):

        candidate = form.get_document()
        parameters = form.get_parameters()
        fields = trans.get_fields_from_path()
        name = fields.get("name", [u"None"])[0]

        # Get the "show" resource path.
        # NOTE: This should be obtained from the site map.

        vpath = trans.get_processed_virtual_path_info()
        show_path = trans.get_path_without_info() + trans.update_path(vpath, "show")
        admin_path = trans.get_path_without_info() + trans.update_path(vpath, "")

        # Redirect if the CV is to be shown.

        if parameters.has_key("show"):

            # Save the candidate information.

            self.repository["candidate-%s" % name] = candidate.toString()
            trans.redirect(trans.encode_path(show_path) +
                "?name=%s" % trans.encode_path(name))

        # Redirect if the administration interface is to be used.

        elif parameters.has_key("admin"):

            # Save the candidate information.

            self.repository["candidate-%s" % name] = candidate.toString()
            trans.redirect(trans.encode_path(admin_path))

# Site map initialisation.

def get_site(fsencoding=None, use_cwd=0):

    "Return a simple Web site resource."

    if use_cwd:
        resource_dir = os.getcwd()
    else:
        resource_dir = resources(__file__)
    repository = DirectoryRepository(os.path.join(resource_dir, "candidates"), fsencoding)

    # Get the main resource and the directory used by the application.

    candidate_resource = CandidateResource(repository)
    display_resource = DisplayResource(repository)
    admin_resource = AdminResource(repository)

    # Make a simple Web site.

    resource = MapResource({
        "edit" : candidate_resource,
        "show" : display_resource,
        "" : admin_resource
        })

    return EncodingSelector(resource, encoding)

# Resource preparation ahead of time - useful for making installations.

def prepare_resources():
    for cls in [AdminResource, DisplayResource, CandidateResource]:
        xslforms_prepare_resources(cls)

# vim: tabstop=4 expandtab shiftwidth=4
